@@ -4,7 +4,7 @@ exec ruby -x "$0" "$@"
 #!ruby
 =begin rdoc
 = mikutter - simple, powerful and moeful Mastodon client
-Copyright (C) 2009-2021 Toshiaki Asai
+Copyright (C) 2009-2022 Toshiaki Asai
 
 This software is released under the MIT License.
 
@@ -43,7 +43,6 @@ require 'lib/weakstorage'
 
 require_relative 'core/utils'
 
-require_if_exist 'webrick'
 require 'boot/check_config_permission'
 require 'boot/mainloop'
 require 'boot/delayer'
@@ -61,7 +60,6 @@ Plugin.call(:boot, nil)
 # _profile_ がtrueなら、プロファイリングした結果を一時ディレクトリに保存する
 def boot!(profile)
   begin
-    Mainloop.before_mainloop
     if profile
       require 'ruby-prof'
       begin
