@@ -9,10 +9,7 @@ require_relative 'util'
 require_relative 'api'
 require_relative 'parser'
 require_relative 'model/model'
-require_relative 'patch'
 require_relative 'spell'
-require_relative 'setting'
-require_relative 'subparts_status_info'
 require_relative 'extractcondition'
 require_relative 'score'
 
@@ -83,10 +80,16 @@ Plugin.create(:mastodon) do
     Delayer.new(delay: 10 * HYDE, &followings_updater) # 26分ごとにプロフィールとフォロー一覧を更新する
   end
 
+  # worldのロード完了時の処理
+  world_load_listener = on_mastodon_worlds__add do
+    world_load_listener.detach
+    Delayer.new { followings_updater.call }
+  end
+
   # 起動時
-  Delayer.new {
+  Delayer.new do
     followings_updater.call
-  }
+  end
 
   # Mastodonサーバが初期化されたら、サーバの集合に加える
   collection(:mastodon_servers) do |servers|
@@ -105,6 +108,7 @@ Plugin.create(:mastodon) do
         a | worlds.flat_map do |world|
           [
             world.sse.user,
+            world.sse.mention,
             world.sse.direct,
             world.sse.public,
             world.sse.public(only_media: true),
@@ -157,6 +161,14 @@ Plugin.create(:mastodon) do
     [world]
   end
 
+  on_mastodon_server_created do |server|
+    Plugin::Mastodon::API.call(
+      :get, server.domain, '/api/v1/instance'
+    ).next do |response|
+      server.configuration = response[:configuration]
+    end
+  end
+  
   # サーバー編集
   on_mastodon_update_instance do |domain|
     Thread.new {
@@ -192,7 +204,12 @@ Plugin.create(:mastodon) do
       Plugin.call(:extract_tab_create, {
                     name: _('ホームタイムライン (%{name})') % name_param,
                     slug: htl_slug,
-                    sources: [world.sse.user.datasource_slug],
+                    sources: Set[ # restとsseではslugが重複するが、ソースとする範囲を表現するため、明示する
+                      world.rest.user.datasource_slug,
+                      world.sse.user.datasource_slug,
+                      world.rest.mention.datasource_slug,
+                      world.sse.mention.datasource_slug,
+                    ],
                     icon: Skin[:timeline].uri,
                   })
     end
@@ -200,8 +217,10 @@ Plugin.create(:mastodon) do
       Plugin.call(:extract_tab_create, {
                     name: _('メンション (%{name})') % name_param,
                     slug: mention_slug,
-                    sources: [:mastodon_appear_toots],
-                    sexp: [:or, [:include?, :receiver_idnames, world.user_obj.idname]],
+                    sources: Set[
+                      world.rest.mention.datasource_slug,
+                      world.sse.mention.datasource_slug,
+                    ],
                     icon: Skin[:reply].uri,
                   })
     end
@@ -209,7 +228,10 @@ Plugin.create(:mastodon) do
       Plugin.call(:extract_tab_create, {
                     name: _('ローカルタイムライン (%{domain})') % name_param,
                     slug: ltl_slug,
-                    sources: [world.sse.public_local.datasource_slug],
+                    sources: Set[
+                      world.rest.public_local.datasource_slug,
+                      world.sse.public_local.datasource_slug,
+                    ],
                     icon: 'https://%{domain}/apple-touch-icon.png' % slug_param,
                   })
     end
@@ -236,15 +258,18 @@ Plugin.create(:mastodon) do
              'mstdn.nere9.help' => _('nere9'),
              'mstdn.y-zu.org' => _('Yづドン'),
             ) do
-        option(:other, _('その他')) do
-          input _('ドメイン'), :domain
+        option :other do
+          input _('その他'), :domain
         end
       end
 
       result = await_input
       domain = result[:domain_selection] == :other ? result[:domain] : result[:domain_selection]
 
-      instance = await Plugin::Mastodon::Instance.add_ifn(domain).trap{ nil }
+      instance = await Plugin::Mastodon::Instance.add_ifn(domain).trap{ |err|
+        error err
+        nil
+      }
       unless instance
         error_msg = _("%{domain} サーバーへの接続に失敗しました。やり直してください。") % {domain: domain}
         next
@@ -258,9 +283,15 @@ Plugin.create(:mastodon) do
       if error_msg.is_a? String
         label error_msg
       end
-      label _('Webページにアクセスして表示された認証コードを入力して、次へボタンを押してください。')
-      link instance.authorize_url
-      input _('認証コード'), :authorization_code
+
+      s = _('認証ページ')
+      # HTML escape
+      url = instance.authorize_url.gsub '&', '&amp;'
+      text = _('認証ページにアクセスして、表示されたコードを貼り付け、「進む」ボタンを押してください。')
+        .sub s, "<a href=\"#{url}\">#{s}</a>"
+      markup text
+
+      input _('認証コード'), :authorization_code, :paste
       if error_msg.is_a? String
         input _('アクセストークンがあれば入力してください'), :access_token
       end
@@ -291,7 +322,7 @@ Plugin.create(:mastodon) do
       if !resp || resp.value.has_key?(:error)
         label _('認証に失敗しました。') + (resp && resp[:error] ? "：#{resp[:error]}" : '')
         await_input
-        raise (resp ? resp[:error] : 'error has occurred at /oauth/token')
+        raise resp ? resp[:error] : 'error has occurred at /oauth/token'
       end
       token = resp[:access_token]
     else
@@ -301,7 +332,7 @@ Plugin.create(:mastodon) do
     resp = Plugin::Mastodon::API.call!(:get, domain, '/api/v1/accounts/verify_credentials', token)
     if resp.nil? || resp.value.has_key?(:error)
       label _('アカウント情報の取得に失敗しました') + (resp && resp[:error] ? "：#{resp[:error]}" : '')
-      raise (resp ? resp[:error] : 'error has occurred at verify_credentials')
+      raise resp ? resp[:error] : 'error has occurred at verify_credentials'
     end
 
     screen_name = resp[:acct] + '@' + domain
